@@ -7,6 +7,7 @@ in the source distribution for its full text.
 
 #include "Object.h"
 #include "RichString.h"
+#include "CRT.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdbool.h>
@@ -51,7 +52,7 @@ void Object_delete(Object* this) {
 void Object_display(Object* this, RichString* out) {
    char objAddress[50];
    sprintf(objAddress, "%s @ %p", this->class, (void*) this);
-   RichString_write(out, A_NORMAL, objAddress);
+   RichString_write(out, CRT_colors[DEFAULT_COLOR], objAddress);
 }
 
 bool Object_equals(const Object* this, const Object* o) {
